import os
import sys
import warnings as _warnings

import click
import IPython

from .config import HissConfig
from .magic import *  # noqa
from .virtualenv import load_venv

_PYTHON_VERSION = "python{0}.{1}".format(*sys.version_info[0:2])
_HISS_CONFIG = os.path.expanduser(os.path.join(os.environ.get('HOME', '~'), '.hiss'))
_BANNER = """
hiss - {python_version}

"""


@click.command()
@click.option('--config', '-c', type=click.Path(exists=False), default=_HISS_CONFIG)
@click.option('--warnings/--no-warnings', default=False)
def main(config, warnings):
    """Console script for hiss"""
    banner = _BANNER.format(python_version=_PYTHON_VERSION)

    if not warnings:
        for warning in (UserWarning, DeprecationWarning, RuntimeWarning):
            _warnings.filterwarnings("ignore", category=warning)

    # load virtualenv & config and run ipython
    load_venv(_PYTHON_VERSION)
    hc = HissConfig(config, banner)
    IPython.start_ipython(
        argv=[],
        config=hc.config,
        quick=True,
        auto_create=False,
        )



import configparser
import os
import six

from IPython.terminal.prompts import ClassicPrompts

from cached_property import cached_property
from traitlets.config import Config as Config
from pygments.styles import get_style_by_name
from pygments.util import ClassNotFound

_BOOLMAP = {
    'true': True,
    't': True,
    '1': True,
    'false': False,
    'f': False,
    '0': False,
}


def _bool(value):
    return value if type(value) == bool else _BOOLMAP[value.lower()]


class HissConfig(object):

    _OVERRIDES = (
        'theme',
        'autoreload',
        'confirm_exit',
        'colors',
        'deep_reload',
        'editor',
        'xmode',
    )

    def __init__(self, path, banner):
        self.path = os.path.expanduser(path)

        # initialize traitlets config obj
        self.config = Config()

        # hardcoded configs
        self.config.TerminalInteractiveShell.prompts_class = ClassicPrompts
        self.config.TerminalInteractiveShell.separate_in = ''
        self.config.TerminalInteractiveShell.true_color = True
        self.config.TerminalInteractiveShell.banner1 = banner
        self.config.PrefilterManager.multi_line_specials = True

        # overrideable configs
        self.theme()
        self.autoreload()
        self.config.TerminalInteractiveShell.confirm_exit = _bool(self.rc.get('confirm_exit', False))
        self.config.TerminalInteractiveShell.autoindent = _bool(self.rc.get('autoindent', True))
        self.config.TerminalInteractiveShell.colors = self.rc.get('colors', 'Linux')

        # ensure magics will be loaded
        self.config.InteractiveShellApp.exec_lines.extend([
            'from hiss.cli.magic import HissMagics',
            'get_ipython().register_magics(HissMagics)',
        ])

    @cached_property
    def rc(self):
        config = six.StringIO()
        config.write('[hiss]\n')

        # attempt load of rc file
        try:
            with open(self.path) as f:
                config.write(f.read().replace('%', '%%'))
        except IOError:
            # no rc file found
            pass

        config.seek(0, os.SEEK_SET)
        cp = configparser.SafeConfigParser()
        cp.readfp(config)

        return dict(cp.items('hiss'))

    def theme(self):
        theme = self.rc.get('theme', 'legacy')

        if theme != 'legacy':
            try:
                pygments_style = get_style_by_name(theme)
            except ClassNotFound:
                pygments_style = False
                theme = 'legacy'
        else:
            pygments_style = None

        self.config.TerminalInteractiveShell.highlighting_style = theme

        if pygments_style:
            self.config.TerminalInteractiveShell.highlighting_style_overrides = pygments_style.styles

    def autoreload(self):
        ar = _bool(self.rc.get('autoreload', False))
        if ar:
            self.config.InteractiveShellApp.extensions = ['autoreload']
            self.config.InteractiveShellApp.exec_lines = ['%autoreload 2']



import os
import sys

from IPython.core.magic import Magics, magics_class, line_magic
from IPython import start_ipython


@magics_class
class HissMagics(Magics):
    @line_magic
    def pex(self, entry_point):
        if entry_point:
            sys.path[0] = os.path.abspath(sys.path[0])
            sys.path.insert(0, entry_point)
            sys.path.insert(0, os.path.abspath(os.path.join(entry_point, '.bootstrap')))
            from _pex import pex_bootstrapper
            pex_bootstrapper.bootstrap_pex_env(entry_point)
            self.reload_pkg_resources('')
            print("Bootstrapped into pex {0}.".format(entry_point))
        else:
            print("No pex provided! Doing nothing.")

    @line_magic
    def reload_pkg_resources(self, line):
        import pkg_resources
        import six
        six.moves.reload_module(pkg_resources)

    @line_magic
    def debug(self, line):
        import logging
        logging.basicConfig(level=logging.DEBUG)
        log = logging.getLogger(__name__)
        print("Debug logging enabled.")

    @line_magic
    def flask(self, app):
        try:
            # we assume we have access to flask/werkzeug,
            # if not, go ahead and raise the importerror
            from flask import _request_ctx_stack
            from werkzeug import import_string
        except ImportError as e:
            raise ImportError("You don't have flask/werkzeug! {err}".format(err=e))
        else:
            app = import_string(app)
            with app.test_request_context():
                banner = (
                    "\nNow using flask context from: {app_repr}\n"
                    "There is no escape! Exit the session when you are done.\n".format(
                        app_repr=repr(app)
                    )
                )

                # add our custom message
                get_ipython().banner1 = banner

                # starting a new session dorks up history, so end our sesh
                get_ipython().history_manager.end_session()
                get_ipython().history_manager.reset()

                # respawn ipython with the flask context
                start_ipython(user_ns=dict(app=_request_ctx_stack.top.app))



import os
import site

from .magic import HissMagics


def load_venv(python_version):
    if 'VIRTUAL_ENV' in os.environ:
        virtual_env = os.path.join(
            os.environ.get('VIRTUAL_ENV'),
            'lib',
            python_version,
            'site-packages',
        )
        if os.path.exists(virtual_env):
            site.addsitedir(virtual_env)
            HissMagics().reload_pkg_resources('')
            print("\x1B[3mvirtualenv detected -> {}\x1B[23m".format(virtual_env))



import os

import click
from click.testing import CliRunner

from hiss.cli import main


def test_hello_world():
    runner = CliRunner()
    result = runner.invoke(main)
    assert result.exit_code == 0



import os
import sys
import setuptools
import subprocess

requirements = [
    'Click==6.6',
    'IPython==5.1.0',
    'cached_property==1.2.0',
    'traitlets',
    'configparser',
    'pygments',
    'six',
]

if sys.version_info[0] < 3:
    requirements.extend([
        'enum34',
        'backports.shutil-get-terminal-size',
    ])


class Venv(setuptools.Command):
    user_options = []

    def initialize_options(self):
        """Abstract method that is required to be overwritten"""

    def finalize_options(self):
        """Abstract method that is required to be overwritten"""

    def run(self):
        venv_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'venv', 'hiss')
        venv_cmd = [
            'virtualenv',
            venv_path
        ]
        print('Creating virtual environment in ', venv_path)
        subprocess.check_call(venv_cmd)
        print('Linking `activate` to top level of project.\n')
        print('To activate, simply run `source activate`.')
        try:
            os.symlink(
                os.path.join(venv_path, 'bin', 'activate'),
                os.path.join(os.path.dirname(os.path.abspath(__file__)), 'activate')
            )
        except OSError:
            print('Unable to create symlink, you may have a stale symlink from a previous invocation.')


setuptools.setup(
    name='hiss-repl',
    version='2.0.22',
    description="A simple and easily configured iPython-based python repl",
    author="Loren Carvalho",
    author_email='me@loren.pizza',
    url='https://github.com/sixninetynine/hiss',
    packages=setuptools.find_packages('src'),
    package_dir={'': 'src'},
    entry_points={
        'console_scripts': [
            'hiss=hiss.cli:main'
        ],
    },
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    keywords='hiss',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    cmdclass={'venv': Venv},
)
